// -----------------------------------------------------------------------------
//    ASPiK Plugin Kernel File:  plugincore.cpp
//
/**
    \file   plugincore.cpp
    \author Will Pirkle
    \date   17-September-2018
    \brief  Implementation file for PluginCore object
    		- http://www.aspikplugins.com
    		- http://www.willpirkle.com
*/
// -----------------------------------------------------------------------------
#include "plugincore.h"
#include "plugindescription.h"

/**
\brief PluginCore constructor is launching pad for object initialization

Operations:
- initialize the plugin description (strings, codes, numbers, see initPluginDescriptors())
- setup the plugin's audio I/O channel support
- create the PluginParameter objects that represent the plugin parameters (see FX book if needed)
- create the presets
*/
PluginCore::PluginCore()
{
    // --- describe the plugin; call the helper to init the static parts you setup in plugindescription.h
    initPluginDescriptors();

    // --- default I/O combinations
	// --- for FX plugins
	if (getPluginType() == kFXPlugin)
	{
		addSupportedIOCombination({ kCFMono, kCFMono });
		addSupportedIOCombination({ kCFMono, kCFStereo });
		addSupportedIOCombination({ kCFStereo, kCFStereo });
	}
	else // --- synth plugins have no input, only output
	{
		addSupportedIOCombination({ kCFNone, kCFMono });
		addSupportedIOCombination({ kCFNone, kCFStereo });
	}

	// --- for sidechaining, we support mono and stereo inputs; auxOutputs reserved for future use
	addSupportedAuxIOCombination({ kCFMono, kCFNone });
	addSupportedAuxIOCombination({ kCFStereo, kCFNone });

	// --- create the parameters
    initPluginParameters();

    // --- create the presets
    initPluginPresets();
}

/**
\brief create all of your plugin parameters here

\return true if parameters were created, false if they already existed
*/
bool PluginCore::initPluginParameters()
{
	if (pluginParameterMap.size() > 0)
		return false;

    // --- Add your plugin parameter instantiation code bewtween these hex codes
	// **--0xDEA7--**


	// --- Declaration of Plugin Parameter Objects 
	PluginParameter* piParam = nullptr;

	// --- continuous control: Sum Level
	piParam = new PluginParameter(controlID::sum_level, "Sum Level", "dB", controlVariableType::kDouble, -12.000000, 2.000000, -4.500000, taper::kLinearTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&sum_level, boundVariableType::kDouble);
	addPluginParameter(piParam);

	// --- continuous control: LFF Fc
	piParam = new PluginParameter(controlID::hpf_fc, "LFF Fc", "Hz", controlVariableType::kDouble, 10.000000, 2000.000000, 700.000000, taper::kVoltOctaveTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&hpf_fc, boundVariableType::kDouble);
	addPluginParameter(piParam);

	// --- continuous control: Diff Level
	piParam = new PluginParameter(controlID::diff_level, "Diff Level", "dB", controlVariableType::kDouble, -12.000000, 2.000000, -4.500000, taper::kLinearTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&diff_level, boundVariableType::kDouble);
	addPluginParameter(piParam);

	// --- continuous control: APF Fc
	piParam = new PluginParameter(controlID::apf_fc, "APF Fc", "Hz", controlVariableType::kDouble, 20.000000, 20480.000000, 20480.000000, taper::kVoltOctaveTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&apf_fc, boundVariableType::kDouble);
	addPluginParameter(piParam);

	// --- continuous control: HSF Fc
	piParam = new PluginParameter(controlID::hsf_fc, "HSF Fc", "Hz", controlVariableType::kDouble, 1000.000000, 20480.000000, 5000.000000, taper::kVoltOctaveTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&hsf_fc, boundVariableType::kDouble);
	addPluginParameter(piParam);

	// --- continuous control: LPEQ Fc
	piParam = new PluginParameter(controlID::peq_fc, "LPEQ Fc", "Hz", controlVariableType::kDouble, 20.000000, 10000.000000, 175.000000, taper::kVoltOctaveTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&peq_fc, boundVariableType::kDouble);
	addPluginParameter(piParam);

	// --- continuous control: LPEG Gain
	piParam = new PluginParameter(controlID::peq_gain, "LPEG Gain", "dB", controlVariableType::kDouble, -12.000000, 12.480000, 0.000000, taper::kLinearTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&peq_gain, boundVariableType::kDouble);
	addPluginParameter(piParam);

	// --- continuous control: HSF Gain
	piParam = new PluginParameter(controlID::hsf_gain, "HSF Gain", "dB", controlVariableType::kDouble, -6.000000, 10.000000, 0.000000, taper::kLinearTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&hsf_gain, boundVariableType::kDouble);
	addPluginParameter(piParam);

	// --- continuous control: APF Gain
	piParam = new PluginParameter(controlID::apf_gain, "APF Gain", "dB", controlVariableType::kDouble, -60.000000, 20.000000, -13.000000, taper::kLinearTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&apf_gain, boundVariableType::kDouble);
	addPluginParameter(piParam);

	// --- continuous control: Output Lvl
	piParam = new PluginParameter(controlID::output_gain, "Output Lvl", "dB", controlVariableType::kDouble, -9.000000, 5.000000, 0.000000, taper::kLinearTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&output_gain, boundVariableType::kDouble);
	addPluginParameter(piParam);

	// --- discrete control: Enable APF
	piParam = new PluginParameter(controlID::enable_apf, "Enable APF", "SWITCH OFF,SWITCH ON", "SWITCH ON");
	piParam->setBoundVariable(&enable_apf, boundVariableType::kInt);
	piParam->setIsDiscreteSwitch(true);
	addPluginParameter(piParam);

	// --- discrete control: EQ Listen
	piParam = new PluginParameter(controlID::enable_eqlisten, "EQ Listen", "SWITCH OFF,SWITCH ON", "SWITCH OFF");
	piParam->setBoundVariable(&enable_eqlisten, boundVariableType::kInt);
	piParam->setIsDiscreteSwitch(true);
	addPluginParameter(piParam);

	// --- continuous control: LPEQ Q
	piParam = new PluginParameter(controlID::peq_q, "LPEQ Q", "", controlVariableType::kFloat, 0.200000, 20.000000, 2.750000, taper::kLinearTaper);
	piParam->setParameterSmoothing(true);
	piParam->setSmoothingTimeMsec(100.00);
	piParam->setBoundVariable(&peq_q, boundVariableType::kFloat);
	addPluginParameter(piParam);

	// --- Aux Attributes
	AuxParameterAttribute auxAttribute;

	// --- RAFX GUI attributes
	// --- controlID::sum_level
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483648);
	setParamAuxAttribute(controlID::sum_level, auxAttribute);

	// --- controlID::hpf_fc
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483650);
	setParamAuxAttribute(controlID::hpf_fc, auxAttribute);

	// --- controlID::diff_level
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483652);
	setParamAuxAttribute(controlID::diff_level, auxAttribute);

	// --- controlID::apf_fc
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483648);
	setParamAuxAttribute(controlID::apf_fc, auxAttribute);

	// --- controlID::hsf_fc
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483648);
	setParamAuxAttribute(controlID::hsf_fc, auxAttribute);

	// --- controlID::peq_fc
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483658);
	setParamAuxAttribute(controlID::peq_fc, auxAttribute);

	// --- controlID::peq_gain
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483648);
	setParamAuxAttribute(controlID::peq_gain, auxAttribute);

	// --- controlID::hsf_gain
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483696);
	setParamAuxAttribute(controlID::hsf_gain, auxAttribute);

	// --- controlID::apf_gain
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483648);
	setParamAuxAttribute(controlID::apf_gain, auxAttribute);

	// --- controlID::output_gain
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483648);
	setParamAuxAttribute(controlID::output_gain, auxAttribute);

	// --- controlID::enable_apf
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(1073741824);
	setParamAuxAttribute(controlID::enable_apf, auxAttribute);

	// --- controlID::enable_eqlisten
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(1073741829);
	setParamAuxAttribute(controlID::enable_eqlisten, auxAttribute);

	// --- controlID::peq_q
	auxAttribute.reset(auxGUIIdentifier::guiControlData);
	auxAttribute.setUintAttribute(2147483648);
	setParamAuxAttribute(controlID::peq_q, auxAttribute);


	// **--0xEDA5--**
   
    // --- BONUS Parameter
    // --- SCALE_GUI_SIZE
    PluginParameter* piParamBonus = new PluginParameter(SCALE_GUI_SIZE, "Scale GUI", "tiny,small,medium,normal,large,giant", "normal");
    addPluginParameter(piParamBonus);

	// --- create the super fast access array
	initPluginParameterArray();

    return true;
}

/**
\brief initialize object for a new run of audio; called just before audio streams

Operation:
- store sample rate and bit depth on audioProcDescriptor - this information is globally available to all core functions
- reset your member objects here

\param resetInfo structure of information about current audio format

\return true if operation succeeds, false otherwise
*/



bool PluginCore::reset(ResetInfo& resetInfo)
{
    // --- save for audio processing
    //audioProcDescriptor.sampleRate = resetInfo.sampleRate;
    //audioProcDescriptor.bitDepth = resetInfo.bitDepth;

	//LEFT HI-PASS
	AudioFilterParameters params_hpL = LeftHiPass.getParameters(); //defining params
	params_hpL.algorithm = filterAlgorithm::kHPF1; //defining it as Hi-Pass
	LeftHiPass.setParameters(params_hpL); //set these parameters

	//reset it
	LeftHiPass.reset(resetInfo.sampleRate);

	//RIGHT HI-PASS
	AudioFilterParameters params_hpR = RightHiPass.getParameters(); //defining params
	params_hpR.algorithm = filterAlgorithm::kHPF1; //defining it as Hi-Pass
	RightHiPass.setParameters(params_hpR); //set these parameters

	//reset it
	RightHiPass.reset(resetInfo.sampleRate);

	//Parametric EQ
	AudioFilterParameters params_peq = RightHiPass.getParameters(); //defining params
	params_peq.algorithm = filterAlgorithm::kCQParaEQ; //defining it as a Parametric EQ
	ParaEQ.setParameters(params_peq); //set these parameters

	//reset it
	ParaEQ.reset(resetInfo.sampleRate);
	
	//ALL PASS FILTER
	AudioFilterParameters params_apf = AllPass.getParameters(); //defining params
	params_apf.algorithm = filterAlgorithm::kAPF1; //defining it as All pass
	AllPass.setParameters(params_apf); //set these parameters

	//reset it
	AllPass.reset(resetInfo.sampleRate);


	//High Shelf
	AudioFilterParameters params_hsf = HighShelf.getParameters(); //defining params
	params_hsf.algorithm = filterAlgorithm::kHiShelf; //defining it as All pass
	HighShelf.setParameters(params_hsf); //set these parameters

	//reset it
	HighShelf.reset(resetInfo.sampleRate);
	


    // --- other reset inits
   return PluginBase::reset(resetInfo);
}

/**
\brief one-time initialize function called after object creation and before the first reset( ) call

Operation:
- saves structure for the plugin to use; you can also load WAV files or state information here
*/
bool PluginCore::initialize(PluginInfo& pluginInfo)
{
	// --- add one-time init stuff here

	return true;
}

/**
\brief do anything needed prior to arrival of audio buffers

Operation:
- syncInBoundVariables when preProcessAudioBuffers is called, it is *guaranteed* that all GUI control change information
  has been applied to plugin parameters; this binds parameter changes to your underlying variables
- NOTE: postUpdatePluginParameter( ) will be called for all bound variables that are acutally updated; if you need to process
  them individually, do so in that function
- use this function to bulk-transfer the bound variable data into your plugin's member object variables

\param processInfo structure of information about *buffer* processing

\return true if operation succeeds, false otherwise
*/
bool PluginCore::preProcessAudioBuffers(ProcessBufferInfo& processInfo)
{
    // --- sync internal variables to GUI parameters; you can also do this manually if you don't
    //     want to use the auto-variable-binding
    syncInBoundVariables();

    return true;
}

void PluginCore::updateParameters()
{
	//This Lets the FX Objects know which parts of its code is altered by the GUI

	//HI PASS LEFT -- get Parameters
	AudioFilterParameters params_hpL = LeftHiPass.getParameters();
	//modify filter cutoff
	params_hpL.fc = hpf_fc;
	//push changes back to object
	LeftHiPass.setParameters(params_hpL);

	//HI PASS RIGHT 
	AudioFilterParameters params_hpR = RightHiPass.getParameters();
	//modify filter cutoff
	params_hpR.fc = hpf_fc;
	//push changes back to object
	RightHiPass.setParameters(params_hpR);

	//LF PARAMETRIC EQ
	AudioFilterParameters params_peq = ParaEQ.getParameters();
	//modify filter cutoff
	params_peq.fc = peq_fc;
	//Boost/Cut
	params_peq.boostCut_dB = peq_gain;
	//Q CONTROL
	params_peq.Q = peq_q;
	//push changes back to object
	ParaEQ.setParameters(params_peq);

	//ALL PASS FILTER
	AudioFilterParameters params_apf = AllPass.getParameters();
	//modify filter cutoff
	params_apf.fc = apf_fc;
	//Filter Gain/Level
	//params_apf.boostCut_dB = apf_gain;
	//push changes back to object
	AllPass.setParameters(params_apf);

	//HIGH SHELF FILTER
	AudioFilterParameters params_hsf = HighShelf.getParameters();
	//modify filter cutoff
	params_hsf.fc = hsf_fc;
	//Boost/Cut
	params_hsf.boostCut_dB = hsf_gain;
	//push changes back to object
	HighShelf.setParameters(params_hsf);

}


/**
\brief frame-processing method

Operation:
- decode the plugin type - for synth plugins, fill in the rendering code; for FX plugins, delete the if(synth) portion and add your processing code
- note that MIDI events are fired for each sample interval so that MIDI is tightly sunk with audio
- doSampleAccurateParameterUpdates will perform per-sample interval smoothing

\param processFrameInfo structure of information about *frame* processing

\return true if operation succeeds, false otherwise
*/
bool PluginCore::processAudioFrame(ProcessFrameInfo& processFrameInfo)
{
    // --- fire any MIDI events for this sample interval
    processFrameInfo.midiEventQueue->fireMidiEvents(processFrameInfo.currentFrame);

	// --- do per-frame updates; VST automation and parameter smoothing
	doSampleAccurateParameterUpdates();

 
    // --- FX Plugin:
    if(processFrameInfo.channelIOConfig.inputChannelFormat == kCFMono &&
       processFrameInfo.channelIOConfig.outputChannelFormat == kCFMono)
    {
		// --- pass through code: change this with your signal processing
        processFrameInfo.audioOutputFrame[0] = processFrameInfo.audioInputFrame[0];

        return true; /// processed
    }

 
    // --- Stereo-In/Stereo-Out
	else if (processFrameInfo.channelIOConfig.inputChannelFormat == kCFStereo &&
		processFrameInfo.channelIOConfig.outputChannelFormat == kCFStereo)
	{
		// --- pass through code: change this with your signal processing



		//cooking 
		double sumlevel_cooked = pow(10.0, sum_level / 20.0);

		double difflevel_cooked = pow(10.0, diff_level / 20.0);

		double outlevel_cooked = pow(10.0, output_gain / 20.0);

		double apf_level = pow(10.0, apf_gain / 20.0);


		double xnL = processFrameInfo.audioInputFrame[0];
		double xnR = processFrameInfo.audioInputFrame[1];
		//double ynL = 0.0;
		//double ynR = 0.0;


		//Dry / Sum Channels

		double summed_dry = xnL + xnR;
		double dry_dB = summed_dry * sumlevel_cooked;

		//Input High-Passing
		double left_hpf = LeftHiPass.processAudioSample(xnL); //Left Channel
		double right_hpf = RightHiPass.processAudioSample(xnR); //Right Channel

		//Difference Sum
		double diff = left_hpf - right_hpf;

		//Difference Level
		double diff_dB = diff * difflevel_cooked;

		//LF Parametric EQ
		double peq = ParaEQ.processAudioSample(diff_dB);

		//All pass Filter
		//double apf = diff_dB * apf_level;
		if (enable_apf == true)
		{
			apf = AllPass.processAudioSample(diff_dB) * apf_level;
			apf_listen = AllPass.processAudioSample(dry_dB) * apf_level;

		}
		else
		{
			apf = diff_dB * apf_level;
			apf_listen = dry_dB * apf_level;
		}

		// ~All Pass level is taken care of by a Plugin Parameter in update parameters~
		//High Shelf Filter
		double hsf = HighShelf.processAudioSample(diff_dB);


		// Main Output
		double left_out = dry_dB + peq + apf + hsf;
		double right_out = dry_dB - peq - apf - hsf;

		//EQ Listen Variables

		double peq_listen = ParaEQ.processAudioSample(dry_dB);
		double hsf_listen = HighShelf.processAudioSample(dry_dB);


		//Main and EQ listen Output
		if (enable_eqlisten == true)
		{
			left_out = peq_listen + hsf_listen + apf_listen;
			right_out = peq_listen + hsf_listen + apf_listen;

		}

		else
		{
			// Main Output
			left_out = dry_dB + peq + apf + hsf;
			right_out = dry_dB - peq - apf - hsf;
		}
	
		//Connecting my filter and the GUI
		updateParameters();
		
		//Now Scaling the Output by the output gain control
		double ynL = left_out * outlevel_cooked;
		double ynR = right_out * outlevel_cooked;
		
		// pass through code to make the output work
		processFrameInfo.audioOutputFrame[0] = ynL;
		processFrameInfo.audioOutputFrame[1] = ynR;

        return true; /// processed 
    }
	 
    return false; /// NOT processed
}


/**
\brief do anything needed prior to arrival of audio buffers

Operation:
- updateOutBoundVariables sends metering data to the GUI meters

\param processInfo structure of information about *buffer* processing

\return true if operation succeeds, false otherwise
*/
bool PluginCore::postProcessAudioBuffers(ProcessBufferInfo& processInfo)
{
	// --- update outbound variables; currently this is meter data only, but could be extended
	//     in the future
	updateOutBoundVariables();

    return true;
}

/**
\brief update the PluginParameter's value based on GUI control, preset, or data smoothing (thread-safe)

Operation:
- update the parameter's value (with smoothing this initiates another smoothing process)
- call postUpdatePluginParameter to do any further processing

\param controlID the control ID value of the parameter being updated
\param controlValue the new control value
\param paramInfo structure of information about why this value is being udpated (e.g as a result of a preset being loaded vs. the top of a buffer process cycle)

\return true if operation succeeds, false otherwise
*/
bool PluginCore::updatePluginParameter(int32_t controlID, double controlValue, ParameterUpdateInfo& paramInfo)
{
    // --- use base class helper
    setPIParamValue(controlID, controlValue);

    // --- do any post-processing
    postUpdatePluginParameter(controlID, controlValue, paramInfo);

    return true; /// handled
}

/**
\brief update the PluginParameter's value based on *normlaized* GUI control, preset, or data smoothing (thread-safe)

Operation:
- update the parameter's value (with smoothing this initiates another smoothing process)
- call postUpdatePluginParameter to do any further processing

\param controlID the control ID value of the parameter being updated
\param normalizedValue the new control value in normalized form
\param paramInfo structure of information about why this value is being udpated (e.g as a result of a preset being loaded vs. the top of a buffer process cycle)

\return true if operation succeeds, false otherwise
*/
bool PluginCore::updatePluginParameterNormalized(int32_t controlID, double normalizedValue, ParameterUpdateInfo& paramInfo)
{
	// --- use base class helper, returns actual value
	double controlValue = setPIParamValueNormalized(controlID, normalizedValue, paramInfo.applyTaper);

	// --- do any post-processing
	postUpdatePluginParameter(controlID, controlValue, paramInfo);

	return true; /// handled
}

/**
\brief perform any operations after the plugin parameter has been updated; this is one paradigm for
	   transferring control information into vital plugin variables or member objects. If you use this
	   method you can decode the control ID and then do any cooking that is needed. NOTE: do not
	   overwrite bound variables here - this is ONLY for any extra cooking that is required to convert
	   the GUI data to meaninful coefficients or other specific modifiers.

\param controlID the control ID value of the parameter being updated
\param controlValue the new control value
\param paramInfo structure of information about why this value is being udpated (e.g as a result of a preset being loaded vs. the top of a buffer process cycle)

\return true if operation succeeds, false otherwise
*/
bool PluginCore::postUpdatePluginParameter(int32_t controlID, double controlValue, ParameterUpdateInfo& paramInfo)
{
    // --- now do any post update cooking; be careful with VST Sample Accurate automation
    //     If enabled, then make sure the cooking functions are short and efficient otherwise disable it
    //     for the Parameter involved
    /*switch(controlID)
    {
        case 0:
        {
            return true;    /// handled
        }

        default:
            return false;   /// not handled
    }*/

    return false;
}

/**
\brief has nothing to do with actual variable or updated variable (binding)

CAUTION:
- DO NOT update underlying variables here - this is only for sending GUI updates or letting you
  know that a parameter was changed; it should not change the state of your plugin.

WARNING:
- THIS IS NOT THE PREFERRED WAY TO LINK OR COMBINE CONTROLS TOGETHER. THE PROPER METHOD IS
  TO USE A CUSTOM SUB-CONTROLLER THAT IS PART OF THE GUI OBJECT AND CODE.
  SEE http://www.willpirkle.com for more information

\param controlID the control ID value of the parameter being updated
\param actualValue the new control value

\return true if operation succeeds, false otherwise
*/
bool PluginCore::guiParameterChanged(int32_t controlID, double actualValue)
{
	/*
	switch (controlID)
	{
		case controlID::<your control here>
		{

			return true; // handled
		}

		default:
			break;
	}*/

	return false; /// not handled
}

/**
\brief For Custom View and Custom Sub-Controller Operations

NOTES:
- this is for advanced users only to implement custom view and custom sub-controllers
- see the SDK for examples of use

\param messageInfo a structure containing information about the incoming message

\return true if operation succeeds, false otherwise
*/
bool PluginCore::processMessage(MessageInfo& messageInfo)
{
	// --- decode message
	switch (messageInfo.message)
	{
		// --- add customization appearance here
	case PLUGINGUI_DIDOPEN:
	{
		return false;
	}

	// --- NULL pointers so that we don't accidentally use them
	case PLUGINGUI_WILLCLOSE:
	{
		return false;
	}

	// --- update view; this will only be called if the GUI is actually open
	case PLUGINGUI_TIMERPING:
	{
		return false;
	}

	// --- register the custom view, grab the ICustomView interface
	case PLUGINGUI_REGISTER_CUSTOMVIEW:
	{

		return false;
	}

	case PLUGINGUI_REGISTER_SUBCONTROLLER:
	case PLUGINGUI_QUERY_HASUSERCUSTOM:
	case PLUGINGUI_USER_CUSTOMOPEN:
	case PLUGINGUI_USER_CUSTOMCLOSE:
	case PLUGINGUI_EXTERNAL_SET_NORMVALUE:
	case PLUGINGUI_EXTERNAL_SET_ACTUALVALUE:
	{

		return false;
	}

	default:
		break;
	}

	return false; /// not handled
}


/**
\brief process a MIDI event

NOTES:
- MIDI events are 100% sample accurate; this function will be called repeatedly for every MIDI message
- see the SDK for examples of use

\param event a structure containing the MIDI event data

\return true if operation succeeds, false otherwise
*/
bool PluginCore::processMIDIEvent(midiEvent& event)
{
	return true;
}

/**
\brief (for future use)

NOTES:
- MIDI events are 100% sample accurate; this function will be called repeatedly for every MIDI message
- see the SDK for examples of use

\param vectorJoysickData a structure containing joystick data

\return true if operation succeeds, false otherwise
*/
bool PluginCore::setVectorJoystickParameters(const VectorJoystickData& vectorJoysickData)
{
	return true;
}

/**
\brief use this method to add new presets to the list

NOTES:
- see the SDK for examples of use
- for non RackAFX users that have large paramter counts, there is a secret GUI control you
  can enable to write C++ code into text files, one per preset. See the SDK or http://www.willpirkle.com for details

\return true if operation succeeds, false otherwise
*/
bool PluginCore::initPluginPresets()
{
	// **--0xFF7A--**

	// --- Plugin Presets 
	int index = 0;
	PresetInfo* preset = nullptr;

	// --- Preset: Factory Preset
	preset = new PresetInfo(index++, "Factory Preset");
	initPresetParameters(preset->presetParameters);
	setPresetParameter(preset->presetParameters, controlID::sum_level, -4.500000);
	setPresetParameter(preset->presetParameters, controlID::hpf_fc, 700.000000);
	setPresetParameter(preset->presetParameters, controlID::diff_level, -4.500000);
	setPresetParameter(preset->presetParameters, controlID::apf_fc, 20480.000000);
	setPresetParameter(preset->presetParameters, controlID::hsf_fc, 5000.000000);
	setPresetParameter(preset->presetParameters, controlID::peq_fc, 175.000000);
	setPresetParameter(preset->presetParameters, controlID::peq_gain, 0.000000);
	setPresetParameter(preset->presetParameters, controlID::hsf_gain, 0.000000);
	setPresetParameter(preset->presetParameters, controlID::apf_gain, -13.000000);
	setPresetParameter(preset->presetParameters, controlID::output_gain, 0.000000);
	setPresetParameter(preset->presetParameters, controlID::enable_apf, 1.000000);
	setPresetParameter(preset->presetParameters, controlID::enable_eqlisten, -0.000000);
	setPresetParameter(preset->presetParameters, controlID::peq_q, 2.750000);
	addPreset(preset);

	// --- Preset: INT
	preset = new PresetInfo(index++, "INT");
	initPresetParameters(preset->presetParameters);
	setPresetParameter(preset->presetParameters, controlID::sum_level, -6.720001);
	setPresetParameter(preset->presetParameters, controlID::hpf_fc, 274.235840);
	setPresetParameter(preset->presetParameters, controlID::diff_level, -2.039993);
	setPresetParameter(preset->presetParameters, controlID::apf_fc, 20480.000000);
	setPresetParameter(preset->presetParameters, controlID::hsf_fc, 2146.667480);
	setPresetParameter(preset->presetParameters, controlID::peq_fc, 288.525391);
	setPresetParameter(preset->presetParameters, controlID::peq_gain, 5.040001);
	setPresetParameter(preset->presetParameters, controlID::hsf_gain, 9.599998);
	setPresetParameter(preset->presetParameters, controlID::apf_gain, 0.000000);
	setPresetParameter(preset->presetParameters, controlID::output_gain, -0.119995);
	setPresetParameter(preset->presetParameters, controlID::enable_apf, 0.000000);
	setPresetParameter(preset->presetParameters, controlID::enable_eqlisten, 0.000000);
	setPresetParameter(preset->presetParameters, controlID::peq_q, 0.000000);
	addPreset(preset);


	// **--0xA7FF--**

    return true;
}

/**
\brief setup the plugin description strings, flags and codes; this is ordinarily done through the ASPiKreator or CMake

\return true if operation succeeds, false otherwise
*/
bool PluginCore::initPluginDescriptors()
{
    pluginDescriptor.pluginName = PluginCore::getPluginName();
    pluginDescriptor.shortPluginName = PluginCore::getShortPluginName();
    pluginDescriptor.vendorName = PluginCore::getVendorName();
    pluginDescriptor.pluginTypeCode = PluginCore::getPluginType();

	// --- describe the plugin attributes; set according to your needs
	pluginDescriptor.hasSidechain = kWantSidechain;
	pluginDescriptor.latencyInSamples = kLatencyInSamples;
	pluginDescriptor.tailTimeInMSec = kTailTimeMsec;
	pluginDescriptor.infiniteTailVST3 = kVSTInfiniteTail;

    // --- AAX
    apiSpecificInfo.aaxManufacturerID = kManufacturerID;
    apiSpecificInfo.aaxProductID = kAAXProductID;
    apiSpecificInfo.aaxBundleID = kAAXBundleID;  /* MacOS only: this MUST match the bundle identifier in your info.plist file */
    apiSpecificInfo.aaxEffectID = "aaxDeveloper.";
    apiSpecificInfo.aaxEffectID.append(PluginCore::getPluginName());
    apiSpecificInfo.aaxPluginCategoryCode = kAAXCategory;

    // --- AU
    apiSpecificInfo.auBundleID = kAUBundleID;
	apiSpecificInfo.auBundleName = kAUBundleName;   /* MacOS only: this MUST match the bundle identifier in your info.plist file */
    apiSpecificInfo.auBundleName = kAUBundleName;

    // --- VST3
    apiSpecificInfo.vst3FUID = PluginCore::getVSTFUID(); // OLE string format
    apiSpecificInfo.vst3BundleID = kVST3BundleID;/* MacOS only: this MUST match the bundle identifier in your info.plist file */
	apiSpecificInfo.enableVST3SampleAccurateAutomation = kVSTSAA;
	apiSpecificInfo.vst3SampleAccurateGranularity = kVST3SAAGranularity;

    // --- AU and AAX
    apiSpecificInfo.fourCharCode = PluginCore::getFourCharCode();

    return true;
}

// --- static functions required for VST3/AU only --------------------------------------------- //
const char* PluginCore::getPluginBundleName() { return kAUBundleName; }
const char* PluginCore::getPluginName(){ return kPluginName; }
const char* PluginCore::getShortPluginName(){ return kShortPluginName; }
const char* PluginCore::getVendorName(){ return kVendorName; }
const char* PluginCore::getVendorURL(){ return kVendorURL; }
const char* PluginCore::getVendorEmail(){ return kVendorEmail; }
const char* PluginCore::getAUCocoaViewFactoryName(){ return AU_COCOA_VIEWFACTORY_STRING; }
pluginType PluginCore::getPluginType(){ return kPluginType; }
const char* PluginCore::getVSTFUID(){ return kVSTFUID; }
int32_t PluginCore::getFourCharCode(){ return kFourCharCode; }
